@@ -1,32 +1,20 @@
 #! -*- encoding: utf-8 -*-
-import subprocess
+
+# We import command modules inside their respective function bodies
+# for performance reasons, so turn off the lint warning
+# pylint: disable=import-outside-toplevel
+
 import logging
 import os
 import sys
 from typing import Optional
 
 import click
+import pkg_resources
+import requests
 from autoupgrade import Package
 
 from suite_py.__version__ import __version__
-from suite_py.commands.aggregator import Aggregator
-from suite_py.commands.ask_review import AskReview
-from suite_py.commands.bump import Bump
-from suite_py.commands.check import Check
-from suite_py.commands.create_branch import CreateBranch
-from suite_py.commands.deploy import Deploy
-from suite_py.commands.generator import Generator
-from suite_py.commands.id import ID
-from suite_py.commands.ip import IP
-from suite_py.commands.login import Login
-from suite_py.commands.merge_pr import MergePR
-from suite_py.commands.open_pr import OpenPR
-from suite_py.commands.project_lock import ProjectLock
-from suite_py.commands.qa import QA
-from suite_py.commands.release import Release
-from suite_py.commands.secret import Secret
-from suite_py.commands.status import Status
-from suite_py.commands.docker import Docker
 from suite_py.lib import logger
 from suite_py.lib.config import Config
 from suite_py.lib.handler import git_handler as git
@@ -37,23 +25,48 @@ ALLOW_NO_GIT_SUBCOMMAND = ["login", "qa", "aggregator"]
 ALLOW_NO_HOME_SUBCOMMAND = ["login", "qa", "aggregator"]
 
 
+def fetch_latest_version() -> Optional[pkg_resources.packaging.version.Version]:
+    """
+    Fetches the latest version of suite-py as a pkg_resources parsed version
+    Returns None on error
+    """
+    try:
+        pkg_info = requests.get("https://pypi.org/pypi/suite-py/json").json()
+        return pkg_resources.parse_version(pkg_info["info"]["version"])
+    except Exception:
+        logger.warning("Failed to fetch latest version of suite-py!")
+        return None
+
+
 def upgrade_suite_py_if_needed(break_on_missing_package: bool = False) -> None:
     """
     Sometimes, when `suite-py` is launched with a virtual environment active, autoupgrade
     cannot see the package as installed (as the command is used from the "global" user environment)
     and raises an exception.
     """
+
     try:
-        # This checks that suite-py is available in the current environment
-        subprocess.run(["pip3", "show", "suite-py"], check=True, capture_output=True)
-        # If available, upgrade (if needed)
-        Package("suite_py").upgrade()
-    except Exception as error:
+        pkg_resources.get_distribution("suite_py")
+    except pkg_resources.DistributionNotFound as error:
+        # We are in a virtual environment where suite-py is not installed, don't bother trying to upgrade
+        if break_on_missing_package:
+            raise error
+
         logger.warning(
             "Skipping Suite-Py autoupgrade because the package was not found in the current environment."
         )
-        if break_on_missing_package:
-            raise error
+        return
+
+    current_version = pkg_resources.parse_version(__version__)
+
+    try:
+        # If available, upgrade (if needed)
+        latest_version = fetch_latest_version()
+        if latest_version is None or latest_version > current_version:
+            # If we fail to fetch the latest version, fallback to attempting the upgrade anyway
+            Package("suite_py").upgrade()
+    except Exception as error:
+        logger.warning(f"An error occurred whilst trying to upgrade suite-py: {error}")
 
 
 @click.group()
@@ -126,6 +139,8 @@ def main(ctx, project, timeout, verbose):
 @click.option("--project", required=False, type=str)
 @click.pass_obj
 def bump(obj, project: Optional[str] = None):
+    from suite_py.commands.bump import Bump
+
     Bump(obj["project"], obj["config"], obj["tokens"]).run(project=project)
 
 
@@ -135,6 +150,8 @@ def bump(obj, project: Optional[str] = None):
 @click.option("--card", type=click.STRING, help="YouTrack card number (ex. PRIMA-123)")
 @click.pass_obj
 def cli_create_branch(obj, card):
+    from suite_py.commands.create_branch import CreateBranch
+
     CreateBranch(obj["project"], card, obj["config"], obj["tokens"]).run()
 
 
@@ -144,6 +161,8 @@ def cli_create_branch(obj, card):
 )
 @click.pass_obj
 def cli_lock_project(obj, environment):
+    from suite_py.commands.project_lock import ProjectLock
+
     ProjectLock(obj["project"], environment, "lock", obj["config"]).run()
 
 
@@ -153,18 +172,24 @@ def cli_lock_project(obj, environment):
 )
 @click.pass_obj
 def cli_unlock_project(obj, environment):
+    from suite_py.commands.project_lock import ProjectLock
+
     ProjectLock(obj["project"], environment, "unlock", obj["config"]).run()
 
 
 @main.command("open-pr", help="Open a PR on GitHub")
 @click.pass_obj
 def cli_open_pr(obj):
+    from suite_py.commands.open_pr import OpenPR
+
     OpenPR(obj["project"], obj["config"], obj["tokens"]).run()
 
 
 @main.command("ask-review", help="Requests a PR review")
 @click.pass_obj
 def cli_ask_review(obj):
+    from suite_py.commands.ask_review import AskReview
+
     AskReview(obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -173,6 +198,8 @@ def cli_ask_review(obj):
 )
 @click.pass_obj
 def cli_merge_pr(obj):
+    from suite_py.commands.merge_pr import MergePR
+
     MergePR(obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -185,6 +212,8 @@ def release():
 @click.option("--deploy", is_flag=True, help="Trigger deploy after release creation")
 @click.pass_obj
 def cli_release_create(obj, deploy):
+    from suite_py.commands.release import Release
+
     Release(
         "create", obj["project"], obj["config"], obj["tokens"], flags={"deploy": deploy}
     ).run()
@@ -193,18 +222,24 @@ def cli_release_create(obj, deploy):
 @release.command("deploy", help="Deploy a github release")
 @click.pass_obj
 def cli_release_deploy(obj):
+    from suite_py.commands.release import Release
+
     Release("deploy", obj["project"], obj["config"], obj["tokens"]).run()
 
 
 @release.command("rollback", help="Rollback a deployment")
 @click.pass_obj
 def cli_release_rollback(obj):
+    from suite_py.commands.release import Release
+
     Release("rollback", obj["project"], obj["config"], obj["tokens"]).run()
 
 
 @main.command("deploy", help="Deploy master branch in production")
 @click.pass_obj
 def cli_deploy(obj):
+    from suite_py.commands.deploy import Deploy
+
     Deploy(obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -216,24 +251,32 @@ def docker():
 @docker.command("release", help="Release new docker image")
 @click.pass_obj
 def cli_docker_release(obj):
+    from suite_py.commands.docker import Docker
+
     Docker("release", obj["project"], obj["config"], obj["tokens"]).run()
 
 
 @docker.command("versions", help="List all available versions of specific image")
 @click.pass_obj
 def cli_docker_versions(obj):
+    from suite_py.commands.docker import Docker
+
     Docker("versions", obj["project"], obj["config"], obj["tokens"]).run()
 
 
 @main.command("status", help="Current status of a project")
 @click.pass_obj
 def cli_status(obj):
+    from suite_py.commands.status import Status
+
     Status(obj["project"], obj["config"]).run()
 
 
 @main.command("check", help="Verify authorisations for third party services")
 @click.pass_obj
 def cli_check(obj):
+    from suite_py.commands.check import Check
+
     Check(obj["config"], obj["tokens"]).run()
 
 
@@ -241,6 +284,8 @@ def cli_check(obj):
 @click.argument("environment", type=click.Choice(("staging", "production")))
 @click.pass_obj
 def cli_id(obj, environment):
+    from suite_py.commands.id import ID
+
     ID(obj["project"], obj["config"], environment).run()
 
 
@@ -248,12 +293,16 @@ def cli_id(obj, environment):
 @click.argument("environment", type=click.Choice(("staging", "production")))
 @click.pass_obj
 def cli_ip(obj, environment):
+    from suite_py.commands.ip import IP
+
     IP(obj["project"], obj["config"], environment).run()
 
 
 @main.command("generator", help="Generate different files from templates")
 @click.pass_obj
 def cli_generator(obj):
+    from suite_py.commands.generator import Generator
+
     Generator(obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -282,18 +331,24 @@ def aggregator(ctx, show_list, change):
 @aggregator.command("list", help="List all aggregators with the current record")
 @click.pass_obj
 def cli_aggregator_list(obj):
+    from suite_py.commands.aggregator import Aggregator
+
     Aggregator(obj["config"], "list").run()
 
 
 @aggregator.command("change", help="Change aggregator record")
 @click.pass_obj
 def cli_aggregator_change(obj):
+    from suite_py.commands.aggregator import Aggregator
+
     Aggregator(obj["config"], "change").run()
 
 
 @main.command("login", help="manage login against Auth0")
 @click.pass_obj
 def login(obj):
+    from suite_py.commands.login import Login
+
     Login(obj["config"]).run()
 
 
@@ -305,6 +360,8 @@ def qa():
 @qa.command("update-quota", help="Update quota in QA for a user")
 @click.pass_obj
 def cli_qa_update_quota(obj):
+    from suite_py.commands.qa import QA
+
     QA("update-quota", obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -314,6 +371,8 @@ def cli_qa_update_quota(obj):
 @click.option("-c", "--card", "card", type=str)
 @click.pass_obj
 def cli_qa_list(obj, user, status, card):
+    from suite_py.commands.qa import QA
+
     QA(
         "list",
         obj["project"],
@@ -327,6 +386,8 @@ def cli_qa_list(obj, user, status, card):
 @click.argument("microservices", nargs=-1, required=True)
 @click.pass_obj
 def cli_qa_create(obj, microservices):
+    from suite_py.commands.qa import QA
+
     QA(
         "create",
         obj["project"],
@@ -341,6 +402,8 @@ def cli_qa_create(obj, microservices):
 @click.argument("microservices", nargs=-1, required=True)
 @click.pass_obj
 def cli_qa_update(obj, qa_hash, microservices):
+    from suite_py.commands.qa import QA
+
     QA(
         "update",
         obj["project"],
@@ -354,6 +417,8 @@ def cli_qa_update(obj, qa_hash, microservices):
 @click.argument("qa_hashes", nargs=-1, required=True)
 @click.pass_obj
 def cli_qa_delete(obj, qa_hashes):
+    from suite_py.commands.qa import QA
+
     QA(
         "delete", obj["project"], obj["config"], obj["tokens"], {"hashes": qa_hashes}
     ).run()
@@ -363,6 +428,8 @@ def cli_qa_delete(obj, qa_hashes):
 @click.argument("qa_hash", required=True)
 @click.pass_obj
 def cli_qa_freeze(obj, qa_hash):
+    from suite_py.commands.qa import QA
+
     QA("freeze", obj["project"], obj["config"], obj["tokens"], {"hash": qa_hash}).run()
 
 
@@ -370,6 +437,8 @@ def cli_qa_freeze(obj, qa_hash):
 @click.argument("qa_hash", required=True)
 @click.pass_obj
 def cli_qa_unfreeze(obj, qa_hash):
+    from suite_py.commands.qa import QA
+
     QA(
         "unfreeze", obj["project"], obj["config"], obj["tokens"], {"hash": qa_hash}
     ).run()
@@ -378,6 +447,8 @@ def cli_qa_unfreeze(obj, qa_hash):
 @qa.command("check", help="Check QA conf")
 @click.pass_obj
 def cli_qa_check(obj):
+    from suite_py.commands.qa import QA
+
     QA("check", obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -386,6 +457,8 @@ def cli_qa_check(obj):
 @click.option("--json", is_flag=True, default=False, help="Get response as JSON")
 @click.pass_obj
 def cli_qa_describe(obj, qa_hash, json):
+    from suite_py.commands.qa import QA
+
     flags = {"hash": qa_hash, "json": json}
     QA("describe", obj["project"], obj["config"], obj["tokens"], flags).run()
 
@@ -396,6 +469,8 @@ def cli_qa_describe(obj, qa_hash, json):
 )
 @click.pass_obj
 def cli_qa_toggle_maintenance(obj):
+    from suite_py.commands.qa import QA
+
     QA("toggle-maintenance", obj["project"], obj["config"], obj["tokens"]).run()
 
 
@@ -410,6 +485,8 @@ def secret():
 @click.option("-b", "--base-profile", "base_profile", required=False)
 @click.pass_obj
 def cli_secret_create(obj, base_profile):
+    from suite_py.commands.secret import Secret
+
     Secret(obj["project"], obj["config"], "create", base_profile).run()
 
 
@@ -417,4 +494,6 @@ def cli_secret_create(obj, base_profile):
 @click.option("-b", "--base-profile", "base_profile", required=False)
 @click.pass_obj
 def cli_secret_grant(obj, base_profile):
+    from suite_py.commands.secret import Secret
+
     Secret(obj["project"], obj["config"], "grant", base_profile).run()
