@@ -9,6 +9,7 @@ from typing import Any, Dict, List, Optional, cast
 from yaml import safe_load
 
 from suite_py.lib import logger
+from suite_py.lib.handler.changelog_handler import ChangelogHandler
 from suite_py.lib.handler.git_handler import GitHandler
 from suite_py.lib.handler.github_handler import GithubHandler
 from suite_py.lib.handler.version_handler import VersionHandler
@@ -21,6 +22,7 @@ class Bump:
 
     # pylint: disable=too-many-instance-attributes
     def __init__(self, project, config, tokens):
+        self._changelog_handler = ChangelogHandler()
         _github = GithubHandler(tokens)
         self._repo = _github.get_repo(project)
         _git = GitHandler(project, config)
@@ -43,6 +45,12 @@ class Bump:
         self._replace_version_in_project_files(
             current_version, cast(str, selected_version), version_config
         )
+
+        if self._changelog_handler.changelog_exists():
+            self._changelog_handler.bump_changelog(
+                self._repo.html_url, current_version, selected_version
+            )
+
         logger.info(f"Successfully bumped to version {selected_version}")
 
     @staticmethod
