@@ -5,10 +5,10 @@ import sys
 from suite_py.commands import common
 from suite_py.commands.release import _parse_available_countries
 from suite_py.lib import logger
-from suite_py.lib import changelog as cl
 from suite_py.lib.handler import git_handler as git
 from suite_py.lib.handler import prompt_utils
 from suite_py.lib.handler.captainhook_handler import CaptainHook
+from suite_py.lib.handler.changelog_handler import ChangelogHandler
 from suite_py.lib.handler.drone_handler import DroneHandler
 from suite_py.lib.handler.git_handler import GitHandler
 from suite_py.lib.handler.github_handler import GithubHandler
@@ -23,6 +23,7 @@ class Deploy:
         self._config = config
         self._youtrack = YoutrackHandler(config, tokens)
         self._captainhook = CaptainHook(config)
+        self._changelog_handler = ChangelogHandler()
         self._github = GithubHandler(tokens)
         self._repo = self._github.get_repo(project)
         self._git = GitHandler(project, config)
@@ -80,8 +81,11 @@ class Deploy:
             new_version = DEFAULT_VERSION
             message = f"First release with tag {new_version}"
 
-        if cl.changelog_exists():
-            latest_tag, latest_entry = cl.get_latest_entry_with_tag()
+        if self._changelog_handler.changelog_exists():
+            (
+                latest_tag,
+                latest_entry,
+            ) = self._changelog_handler.get_latest_entry_with_tag()
 
             if latest_tag != new_version:
                 if not prompt_utils.ask_confirm(
