@@ -65,7 +65,7 @@ class Secret:
         value = prompt_utils.ask_questions_input("Enter secret value: ")
         e = self._vault.exec(
             self._config.vault["base_secret_profile"],
-            f"biscuit put -f config/secrets.yml {secret} {value}",
+            f"biscuit put -f config/secrets.yml -- {secret} {value}",
         )
         if e.returncode != 0:
             logger.error("An error occurred.")
